<?php
  /*
  Plugin Name: Woocommerce Subscriptions Renewal Reminder
  Plugin URI: http://qwebmaster.com/woocommerce-subscriptions-renewal-reminder/
  Description: Reminds Subscription customers X days before the subscription renewal date
  Version: 1.5
  Requires at least: 4.0
  Tested up to: 5.7.2
  WC requires at least: 3.0
  WC tested up to: 5.4.1
  Requires PHP: 7.3
  Author: Qwebmaster
  Author URI: http://qwebmaster.com
  */

  /*
  V1.6
  TODO
  #1 Code to update
  // Getting the related Order ID (added WC 3+ comaptibility)
  $order_id = method_exists( $subscription, 'get_parent_id' ) ? $subscription->get_parent_id() : $subscription->order->id;

  // Getting an instance of the related WC_Order Object (added WC 3+ comaptibility)
  $order = method_exists( $subscription, 'get_parent' ) ? $subscription->get_parent() : $subscription->order;

  #2 Hooks to update
  woocommerce_subscription_status_updated
  To handle switched subscriptions too (upgrade or downgrade a subscription)
  You can try to use use woocommerce_subscription_status_updated related hook.
  In the code below, you will set for each variation ID, the related user role in the settings array (commented code):

  #3 Improvement in efficiency
  Test Using woocommerce_subscription_status_updated to replace 
  add_action( 'woocommerce_subscription_status_cancelled', 'wsrr_subscription_status_updated',10,1);
  add_action( 'woocommerce_subscription_status_expired', 'wsrr_subscription_status_updated',10,1);
  add_action( 'woocommerce_subscription_status_on-hold', 'wsrr_subscription_status_updated',10,1);
  add_action( 'woocommerce_subscription_status_active', 'wsrr_subscription_status_active' );

  TO TEST 
  add_action( 'wsrr_renewal_reminder_email', 'wsrr_renewal_reminder_email', 10, 1 );
  */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Check if WooCommerce is activated
 */
if ( ! function_exists( 'is_woocommerce_activated' ) ) {
  function is_woocommerce_activated() {
    if ( class_exists( 'woocommerce' ) ) { return true; } else { 
      return false;
    }
  }
}

/**
 * Check if Woocommerce Subscriptions is active
 */
if ( !is_plugin_active('woocommerce-subscriptions/woocommerce-subscriptions.php') ) {
  add_action( 'admin_notices', 'wsrr_woocommerce_subscriptions_notice' );
}
function wsrr_woocommerce_subscriptions_notice(){
        echo '<div class="notice notice-warning is-dismissible">
            <p>Woocommerce Subscriptions Renewal Reminder requires the plugin "WooCommerce Subscriptions" and "Woocommerce" to be active.</p>
        </div>';
}

/**
 * Function to send the test email
 */
add_action("wp_ajax_infiniteScroll", 'infiniteScroll');
function infiniteScroll() { 
  $email = $_POST['test_email'];
  $order_id = $_POST['order_id'];
  // Load WC mailer
  $mailer = WC()->mailer()->get_emails();
  $mailer['WC_Customer_Subscription_Renewal_Reminder']->trigger( $order_id, $email );
  die(); 
}

/**
 * Class Subscription_Renewal_Reminder_WC_Email
 */
class Subscription_Renewal_Reminder_WC_Email {

	/**
	 * Custom_WC_Email constructor.
	 */
	public function __construct() {
    // Filtering the emails and adding our own email.
		add_filter( 'woocommerce_email_classes', array( $this, 'register_email' ), 90, 1 );
    // Absolute path to the plugin folder.
		define( 'CUSTOM_WC_EMAIL_PATH', plugin_dir_path( __FILE__ ) );
	}

	/**
	 * @param array $emails
	 *
	 * @return array
	 */
	public function register_email( $emails ) {
		require_once 'emails/class-wsrr-renewal-reminder.php';
		$emails['WC_Customer_Subscription_Renewal_Reminder'] = new WC_Customer_Subscription_Renewal_Reminder();
		return $emails;
	}
}
new Subscription_Renewal_Reminder_WC_Email();

/**
 * Load Admin Settings Class 
 */
require_once( 'wsrr_settings.php' );

/**
 * Load action scheduler v3.1.6
 */
require_once( plugin_dir_path( __FILE__ ) . '/libraries/action-scheduler/action-scheduler.php' );

/**
*  When a subscription order status is changed, reove the email from queue
*
* @since 0.1
*/
function wsrr_subscription_status_updated($subscription){
  // Renewal Date
  $trigger_date = $subscription->get_time( 'next_payment' );
  $end_date = $subscription->get_date( 'end', 'site' );
  if (empty($trigger_date) && !empty($end_date)) {
    $email_body = $options['email_body_cheque'];
    $trigger_date = $subscription->get_date( 'end', 'site' );
    wsrr_logger('End date trigger '.$subscription->get_id());
  }
  $hook = 'wsrr_renewal_reminder_email';
  $args = array(
    'subscription_id' => $subscription->get_id(),
    // Removed V1.3
    //'payment_method' => $subscription->payment_method,
    'payment_method' => $subscription->get_payment_method(),
    //'renewal_date'    => gmdate("Y-m-d\ H:i:s", $trigger_date)
  );
  $group = 'wsrr';
  wsrr_logger(print_r($args, true));
  wsrr_unschedule_scheduled_actions($hook,$args,$group);
}
// When status is set to cancelled
add_action( 'woocommerce_subscription_status_cancelled', 'wsrr_subscription_status_updated',10,1);
// In case subscription expires
add_action( 'woocommerce_subscription_status_expired', 'wsrr_subscription_status_updated',10,1);
// If subscription is set to on hold
add_action( 'woocommerce_subscription_status_on-hold', 'wsrr_subscription_status_updated',10,1);

function wsrr_woocommerce_updated_post_meta($meta_id, $post_id, $meta_key='',$meta_value=''){
    // Stop if not the correct meta key
    if ( $meta_key == '_schedule_next_payment' || $meta_key == '_schedule_end') {
        wsrr_woo_get_all_active_subsctiptions();
    }
}
add_action('updated_post_meta', 'wsrr_woocommerce_updated_post_meta', 10, 4);

/**
 *  When an order is updated (manually) remove the scheduled action
 *
 * @since 0.5
 */
add_action( 'woocommerce_process_shop_order_meta', 'wsrr_woocommerce_process_shop_order', 10, 2 );
function wsrr_woocommerce_process_shop_order ( $post_id, $post ) {
      if ( ! is_admin()) {
        return;
      }
      if ( $post->post_type == 'shop_subscription') {
        $subscription = new WC_Subscription( $post_id );
        // Renewal Date
        // $trigger_date = $subscription->get_time( 'next_payment' );
        //$end_date = $subscription->get_date( 'end', 'site' );
        //if (empty($trigger_date) && !empty($end_date)) {
          //$email_body = $options['email_body_cheque'];
          //$trigger_date = $subscription->get_date( 'end', 'site' );
        //}
        $hook = 'wsrr_renewal_reminder_email';
        $args = array(
          'subscription_id' => $subscription->get_id(),
          'payment_method' => $subscription->get_payment_method(),
        //  'renewal_date'    => gmdate("Y-m-d\ H:i:s", $trigger_date)
        );
        $group = 'wsrr';
        wsrr_unschedule_scheduled_actions($hook,$args,$group);
        //wsrr_woo_get_all_active_subsctiptions();
      }
}

/**
 *  When a subscription order status is set to active run the main hook "wsrr_renewal_reminder_scheduler"
 *
 * @since 0.1
 */
function wsrr_subscription_status_active(){
  wsrr_woo_get_all_active_subsctiptions();
}
add_action( 'woocommerce_subscription_status_active', 'wsrr_subscription_status_active' );

/**
 *  Schedule main wsrr_renewal_reminder_scheduler action
 *
 * @since 0.1
 */
add_action( 'init', 'wsrr_renewal_reminder_scheduler' );
function wsrr_renewal_reminder_scheduler(){
  $options = get_option( 'wsrr_option' );
  // skip email and adding post meta if Debug is checked
  $wsrr_enable = $options['wsrr_enable'] ?? '0'; // default: 0 disabled
  if ($wsrr_enable != 1) {
    // If not enabled
    return;
  }
  // Check if the action is alredy scheduled
  $interval_in_seconds = 24*60*60; // 24h in seconds
  $hook = 'wsrr_renewal_reminder_scheduler';
  $args = array(
    //'subscription_id' => $subscription_id,
  );
  $group = '';
  $params = array();
  if ( is_array($args) ) {
    $params['args'] = $args;
  }
  if ( !empty($group) ) {
    $params['group'] = $group;
  }
  //$action = ActionScheduler::store()->find_action( $hook, $params,$group );
  if (wsrr_check_scheduled_actions($hook,$args,$group)){
     // Action alredy scheduled do nothing
     // wsrr_unschedule_scheduled_actions($hook,$args,$group);
    return;
  } else {
   // Schedule Action
     as_schedule_recurring_action( time(), $interval_in_seconds, $hook, $args, $group );
     wsrr_logger('Scheduled Recurring Action:');
     wsrr_logger('   $interval_in_seconds:'.$interval_in_seconds);
     wsrr_logger('   $hook:'.$hook);
     wsrr_logger('   $args:.'.print_r($args,true));
  }
}

/**
 *  Triggers once every X hours, scans all subscriptions
 *
 * @since 0.1
 * @essential
 */
add_action( 'wsrr_renewal_reminder_scheduler', 'wsrr_renewal_reminder', 10, 1  );
function wsrr_renewal_reminder(){
  wsrr_woo_get_all_active_subsctiptions();
}

/**
*  Trigger when an email is sent out via the Actions Scheduler
*
* @since 1.5
* @param $order_id
*/
function wsrr_renewal_reminder_email( $subscription_id ) {
  $subscription = new WC_Subscription( $subscription_id );
  $options = get_option( 'wsrr_option' );
  // Skip email and adding post meta if Debug is checked
  // if ($options['wsrr_debug'] != 1) {
  // Get WooCommerce email objects
  $mailer = WC()->mailer()->get_emails();
  //var_dump($mailer['WC_Customer_Subscription_Renewal_Reminder']->settings);
  // ["enabled"]=> string(3) "yes" ["subject"]=> string(12) "Test subject" 
  // ["heading"]=> string(18) "Test Email heading" ["additional_content"]=> string(23) "TEST Additional content" ["email_type"]=> string(4) "html" }
  // Assign heading & subject to chosen object
  //$mailer['WC_Customer_Subscription_Renewal_Reminder']->heading = $heading;
  //$mailer['WC_Customer_Subscription_Renewal_Reminder']->settings['heading'] = $heading;
  //$mailer['WC_Customer_Subscription_Renewal_Reminder']->subject = $subject;
  //$mailer['WC_Customer_Subscription_Renewal_Reminder']->settings['subject'] = $subject;
  // To add email content use https://businessbloomer.com/woocommerce-add-extra-content-order-email/
  wsrr_logger('Sending Reminder Email:');
  wsrr_logger('   Subscription ID:'.$subscription_id);
  wsrr_logger('   $subject:'.$mailer['WC_Customer_Subscription_Renewal_Reminder']->subject);
 //$mailer['WC_Customer_Subscription_Renewal_Reminder']->settings2 = 'testhhhhh';
 // Send the renewal reminder email
 $mailer['WC_Customer_Subscription_Renewal_Reminder']->trigger( $subscription_id );
 // Set the payment count to mark this email as sent
 // v1.4 Updated get_completed_payment_count -> get_payment_count
 $completed_paymet_count = $subscription->get_payment_count();
 wsrr_logger('$completed_paymet_count before meta update:'.$completed_paymet_count);
 // ENABLE BEFORE LIVE
 //add_post_meta( $subscription_id, 'subscription_renewal_reminder', 2);
 update_post_meta( $subscription_id, 'subscription_renewal_reminder',$completed_paymet_count);
 wsrr_logger('subscription_renewal_reminder after update:'.get_post_meta( $subscription_id, 'subscription_renewal_reminder', true ));
}
add_action( 'wsrr_renewal_reminder_email', 'wsrr_renewal_reminder_email', 10, 1 );

/**
*  Override the default woocommerce email template
*
* @since 0.1
* @param $order
* @param $heading = false
* @param $mailer
*/
// Disabled v1.5
/*
function wsrr_renewal_reminder_email_template( $order, $heading = false, $mailer, $email_template) {
$options = get_option( 'wsrr_option' );
//$template = 'emails/admin-new-order.php';
$template = 'emails/'.$email_template;
return wc_get_template_html( $template, array(
  'order'         => $order,
  'email_heading' => $options['email_title'],
  'sent_to_admin' => true,
  'plain_text'    => false,
  'email'         => $mailer
),
'',
wsrr_plugin_path() . "/templates/" );
} 
*/

/**
 *  Get plugin path
 *
 * @since 0.1
 * @essential
 */
function wsrr_plugin_path(){
    return untrailingslashit( plugin_dir_path( __FILE__ ) );
}

/**
 *  Get plugin url path
 *
 * @since 1.5
 * @essential
 */
function wsrr_plugin_dir_url(){
  return plugin_dir_url( __FILE__ );
}


/**
*  Event Logger Woocommerce -> Status -> Log
*
* @since 1.5
*/
function wsrr_logger($data){
    // Addev v1.5
    // Debug log in Woocommerce -> Status -> Log
    $options = get_option( 'wsrr_option' );
    if ($options['wsrr_debug'] == 1 ) {
      $logger = wc_get_logger();
      $logger->debug( $data, array( 'source' => 'woocommerce-subscriptions-renewal-reminder' ) );
    }
}

/**
*  Get all Active WooCommerce Subscriptions
*
* @since 0.1
*/
function wsrr_woo_get_all_active_subsctiptions(){
  // Added v1.3
  // Exit if this is the checkout page (performance issue v1.2 <)
  if ( is_checkout() ) {
  return;
  }
  $options = get_option( 'wsrr_option' );
  wsrr_logger('Scanning for active subscriptions...');
  global $wpdb;
  $posts = $wpdb->prefix . "posts";
  // Get all subscriptions ID's
  /*
  $subscriptions_ids = $wpdb->get_col("
    SELECT ID
    FROM $posts
    WHERE post_type LIKE 'shop_subscription'
  ");
  // Optimized v1.3
  */
  $subscriptions_ids = $wpdb->get_col("
    SELECT ID
    FROM $posts
    WHERE post_type LIKE 'shop_subscription'
    AND
    post_status = 'wc-active'
  ");
    wsrr_logger('Subscription ID\'s found:'.print_r($subscriptions_ids, true));
    foreach($subscriptions_ids as $subscription_id){
       // Get an instance of the WC_Subscription object
       $subscription = new WC_Subscription( $subscription_id );
       // Only Active subscriptions
       // 'pending', 'active', 'cancelled', 'expired', 'failing', 'completed'.
       if ( $subscription->get_status() == 'active' ) {
       //$options = get_option( 'wsrr_option' );
        // Removed V1.3
        // $billing_period = $subscription->billing_period;
        $billing_period = $subscription->get_billing_period();
        // Exclude Yearly Subscriptions
        if (!empty($options['exclude_year']) && $options['exclude_year'] != 0 ) {
          if ($billing_period == 'year') {
            wsrr_logger('Excluding Yearly Renewals - Skipping Subscrption ID: '.$subscription_id.'');
            continue;
          }
        }
        // Exclude Monthley Subscriptions
        if (!empty($options['exclude_month']) && $options['exclude_month'] != 0 ) {
          if ($billing_period == 'month') {
            wsrr_logger('Excluding Monthly Renewals - Skipping Subscrption ID: '.$subscription_id.'');
            continue;
          }
        }
        // Exclude Weekley Subscriptions
        if (!empty($options['exclude_week']) && $options['exclude_week'] != 0 ) {
          if ($billing_period == 'week') {
            wsrr_logger('Excluding Weekly Renewals - Skipping Subscrption ID: '.$subscription_id.'');
            continue;
          }
        }
        // Exclude Daily Subscriptions
        if (!empty($options['exclude_day']) && $options['exclude_day'] != 0 ) {
          if ($billing_period == 'day') {
            wsrr_logger('Excluding Daily Renewals - Skipping Subscrption ID: '.$subscription_id.'');
            continue;
          }
        }
        $trigger_date = $subscription->get_time( 'next_payment' );
        if (!$trigger_date ) {
          // If date returns 0 try to get the expiration date
          wsrr_logger('$trigger_date returns 0 try to get the expiration date');
          $trigger_date = $subscription->get_date( 'end', 'site' );
          wsrr_logger('$subscription->get_date:'.$trigger_date);
        }
        if ( !is_numeric($trigger_date) && $trigger_date != '0') {
          $trigger_date = strtotime($trigger_date);
          wsrr_logger('!is_numeric($trigger_date) && $trigger_date != 0 $trigger_date = '.$trigger_date);
        }
        // Check if this is a valid timestamp
        // Some orders return the date and not a UNIX timestamp
        // Make sure it's not a 0
        if ( is_numeric($trigger_date) && (int)$trigger_date == $trigger_date  && $trigger_date != '0') {
          wsrr_logger('is_numeric($trigger_date) && (int)$trigger_date == $trigger_date  && $trigger_date != 0');
        //$options = get_option( 'wsrr_option' );
        // Disabled v1.5
        /*
        // If wsrr_debug_subscription_id is set it will only work for that subscription ID
        if (!empty($options['wsrr_debug_subscription_id']) && $options['wsrr_debug_subscription_id'] != 0 ) {
            if ($subscription_id == $options['wsrr_debug_subscription_id']) {

            } else {
                continue;
            }
        }
        */
          // #1
          // Check if renewal date is within X days of today
          // Offset the renewal ofset another 10 days so actions can be scheduled ahead of time
          $renewal_offset = $options['renewal_days'];
          //$renewal_offset = 1 + 10;
          $trigger = 0;
          // Skip reminders that should have been sent out based on X days.
          if (isset($options['skip_overdue']) && $options['skip_overdue'] == 1) {
            if( $trigger_date < strtotime('+'.$renewal_offset + $options['renewal_days_offset'] .' day') && $trigger_date >= strtotime('+'.$renewal_offset.' day') ) {
              $trigger = 1;
              wsrr_logger('Skip Overdue Setting ON trigger val:'.$trigger);
              wsrr_logger('$trigger_date val:'.$trigger_date);
              }

          } else {
            if( $trigger_date < strtotime('+'.$renewal_offset + $options['renewal_days_offset'] .' day') ) {
            wsrr_logger('Skip Overdue Setting OFF trigger val:'.$trigger);
            wsrr_logger('$trigger_date val:'.$trigger_date);
            wsrr_logger('$renewal_offset val:'.$renewal_offset);
            wsrr_logger('renewal_days_offset val:'.$options['renewal_days_offset']);
              $trigger = 1;

            }
          }

          if( $trigger == 1) {
        // Only orders X Days or MORE
        //if( $trigger_date < strtotime('+'.$renewal_offset + $options['renewal_days_offset'] .' day') && $trigger_date >= strtotime('+'.$renewal_offset.' day') ) {
        // All orders X days or LESS and MORE
        // if( $trigger_date < strtotime('+'.$renewal_offset + $options['renewal_days_offset'] .' day') ) {
          $subscription_id = $subscription->get_id();
          $completed_paymet_count = $subscription->get_payment_count();
          wsrr_logger('Trigger Completed Payment Count:'.$completed_paymet_count);
          $status = get_post_meta( $subscription_id, 'subscription_renewal_reminder', true );
          wsrr_logger('Status/subscription_renewal_reminder:'.$status);
          //Check if this renewal email has alredy been sent
          if ( ! empty( $status ) ) {
            // If an email has alredy been sent for this subscription
            // count the number of completed orders and make sure they don't match the current completed orders
            if ($completed_paymet_count <= $status) {
              // 2021-06-12T23:05:10+00:00 DEBUG Trigger Completed Payment Count:3
              // 2021-06-12T23:05:10+00:00 DEBUG Status/subscription_renewal_reminder:2
              wsrr_logger('$completed_paymet_count <= $status');
              // Email alredy sent skip
              wsrr_logger('Email alredy sent for subscription ID: '.$subscription_id.' Skipping...');
              // Email alredy sent;
              continue;
            }
          }
          // If the scheduled action is less than X days when this plugin is activated
          // Run the action right away
          $timestamp = strtotime('-'.$renewal_offset.' day', $trigger_date);
          $interval_in_seconds = 24*60*60; // 24h in seconds
          $hook = 'wsrr_renewal_reminder_email';
          $args = array(
            'subscription_id' => $subscription_id,
            'payment_method' => $subscription->get_payment_method(),
            //'renewal_date'    => gmdate("Y-m-d\ H:i:s", $trigger_date)
          );
          $group = 'wsrr';

          // #2
          // Check if email is alredy in scheduled actions
          // var_dump(wsrr_check_scheduled_actions($hook,$args).'$$$$$$$');
          // Check if there is an action scheduled for this email
          if (wsrr_check_scheduled_actions($hook,$args,$group)){
            //Action Scheduled
            //wsrr_unschedule_scheduled_actions($hook,$args,$group);
            // Email action alredy set
            wsrr_logger('Email action alredy set for subscription ID: '.$subscription_id);
            continue;
          }

          // #3
          // Proceed to scheduling the renewal email
          // as_schedule_recurring_action( $timestamp, $interval_in_seconds, $hook, $args, $group );
          // wc_unschedule_action( 'sfn_followup_emails', $param, 'fue' );
          // wc_schedule_single_action( $send_on, 'sfn_followup_emails', $param, 'fue' );
            as_schedule_single_action($timestamp, $hook, $args, $group);
            wsrr_logger('Proceed to scheduling the renewal email for subscription ID: '.$subscription_id);
            // Set the payment count  when scheduling the action
            // add_post_meta( $subscription_id, 'subscription_renewal_reminder', $completed_paymet_count, true );
            //var_dump('LESS THAN '.$renewal_offset.' DAYS');
            //echo gmdate("Y-m-d\TH:i:s\Z", strtotime('+9 day'));
        }

        } else {
          wsrr_logger('This is not a valid timestamp: '.$trigger_date);
        }

      } else {
      wsrr_logger('This is an inactive subscription '.$subscription_id);
      wsrr_logger('Inactive subscription status: '.$subscription->get_status());
      }
 }
}

 /**
  *  Check Scheduled Actions
  *
  * @since 0.1
  */
function wsrr_check_scheduled_actions($hook,$args,$group) {
  $params = array();
  if ( is_array($args) ) {
    $params['args'] = $args;
  }
  if ( !empty($group) ) {
    $params['group'] = $group;
  }
  $action = ActionScheduler::store()->find_action( $hook, $params,$group );
  return $action;
}

 /**
  *  Remove Scheduled Actions
  *
  * @since 0.1
  */
function wsrr_unschedule_scheduled_actions($hook,$args,$group) {
  //$hook = 'test_action';
  $params = array();
  if ( is_array($args) ) {
    $params['args'] = $args;
  }
  if ( !empty($group) ) {
    $params['group'] = $group;
  }
  // Updated v1.5
  // $action = wc_unschedule_action( $hook, $param, $group );
  $action = as_unschedule_action( $hook, $params, $group );
  return $action;
}

/**
 *  Human Readable array
 *
 * @since 0.1
 * @debug
 */
  function wsrr_print_array($aArray) {
    echo '<pre>';
    print_r($aArray);
    echo '</pre>';
  }