<?php
/**
 *  Admin Settings Class
 *
 * @since 0.2
 * @debug
 */

class WSRRSettingsPage
{
 /**
  * Holds the values to be used in the fields callbacks
  */
 private $options;

 /**
  * Start up
  */
 public function __construct()
 {
     add_action( 'admin_menu', array( $this, 'add_plugin_page' ) );
     add_action( 'admin_init', array( $this, 'page_init' ) );
 }

 /**
  * Add options page
  */
 public function add_plugin_page()
 {
     add_submenu_page(
     'woocommerce',
     'Subscription Renewal Reminder',
     'Subscription Renewal Reminder',
     'manage_options',
     'wsrr-renewal-reminder',
     array( $this, 'create_admin_page')
   );
 }

 /**
  * Options page callback
  */
 public function create_admin_page()
 {
     // Set class property
     $this->options = get_option( 'wsrr_option' );
     ?>
     <div class="wrap">
         <h1>Woocommerce Renewal Reminder Settings</h1>
         <form method="post" action="options.php">
         <?php
             // This prints out all hidden setting fields
             settings_fields( 'wsrr_option_group' );
             do_settings_sections( 'wsrr-renewal-reminder' );
             submit_button();
         ?>
         </form>
     </div>
     <?php
 }

 /**
  * Register and add settings
  */
 public function page_init()
 {
     register_setting(
         'wsrr_option_group', // Option group
         'wsrr_option', // Option name
         array( $this, 'sanitize' ) // Sanitize
     );

     add_settings_section(
         'wsrr_setting_section', // ID
         'General Settings', // Title
         array( $this, 'print_section_info' ), // Callback
         'wsrr-renewal-reminder' // Page
     );


     add_settings_field(
        'wsrr_enable', // ID
        'Enable Reminder', // Title
        array( $this, 'wsrr_enable_callback' ), // Callback
        'wsrr-renewal-reminder', // Page
        'wsrr_setting_section' // Section
     );



     add_settings_field(
         'renewal_days', // ID
         'Notify Customer "X" Days Before Renewal', // Title
         array( $this, 'wsrr_renewal_days_callback' ), // Callback
         'wsrr-renewal-reminder', // Page
         'wsrr_setting_section' // Section
     );

     add_settings_field(
         'renewal_days_offset', // ID
         '"Scan orders X" ( days ) before the renewal notification', // Title
         array( $this, 'wsrr_renewal_days_offset_callback' ), // Callback
         'wsrr-renewal-reminder', // Page
         'wsrr_setting_section' // Section
     );

     add_settings_field(
        'skip_overdue', // ID
        'Skip Overdue Reminders', // Title
        array( $this, 'skip_overdue_callback' ), // Callback
        'wsrr-renewal-reminder', // Page
        'wsrr_setting_section' // Section
     );

     add_settings_field(
        'exclude_year', // ID
        'Exclude Yearly Renewals', // Title
        array( $this, 'exclude_year_callback' ), // Callback
        'wsrr-renewal-reminder', // Page
        'wsrr_setting_section' // Section
     );

     add_settings_field(
        'exclude_month', // ID
        'Exclude Monthly Renewals', // Title
        array( $this, 'exclude_month_callback' ), // Callback
        'wsrr-renewal-reminder', // Page
        'wsrr_setting_section' // Section
     );

     add_settings_field(
        'exclude_week', // ID
        'Exclude Weekly Renewals', // Title
        array( $this, 'exclude_week_callback' ), // Callback
        'wsrr-renewal-reminder', // Page
        'wsrr_setting_section' // Section
     );

     add_settings_field(
        'exclude_day', // ID
        'Exclude Daily Renewals', // Title
        array( $this, 'exclude_day_callback' ), // Callback
        'wsrr-renewal-reminder', // Page
        'wsrr_setting_section' // Section
     );

     //Disabled in v1.5
/*

     add_settings_section(
         'wsrr_email_setting_section', // ID
         'Email Settings', // Title
         array( $this, 'print_section_info' ), // Callback
         'wsrr-renewal-reminder' // Page
     );

     add_settings_field(
         'email_bcc', // ID
         'BCC', // Title
         array( $this, 'wsrr_bcc_callback' ), // Callback
         'wsrr-renewal-reminder', // Page
         'wsrr_email_setting_section' // Section
     );

     add_settings_field(
         'email_cc', // ID
         'CC', // Title
         array( $this, 'wsrr_cc_callback' ), // Callback
         'wsrr-renewal-reminder', // Page
         'wsrr_email_setting_section' // Section
     );

     add_settings_field(
         'email_subject', // ID
         'Email Subject', // Title
         array( $this, 'email_subject_callback' ), // Callback
         'wsrr-renewal-reminder', // Page
         'wsrr_email_setting_section' // Section
     );

     add_settings_field(
         'email_title', // ID
         'Email Title', // Title
         array( $this, 'email_title_callback' ), // Callback
         'wsrr-renewal-reminder', // Page
         'wsrr_email_setting_section' // Section
     );

     add_settings_field(
         'email_body', // ID
         'Email Body', // Title
         array( $this, 'email_body_callback' ), // Callback
         'wsrr-renewal-reminder', // Page
         'wsrr_email_setting_section' // Section
     );
     */
/*
     add_settings_field(
         'email_body_cheque', // ID
         'Email Body (Cheque)', // Title
         array( $this, 'email_body_cheque_callback' ), // Callback
         'wsrr-renewal-reminder', // Page
         'wsrr_email_setting_section' // Section
     );
*/

// Addev v1.5
add_settings_section(
    'wsrr_sca_setting_section', // ID
    'Scheduled Emails', // Title
    array( $this, 'print_section_footer' ), // Callback
    'wsrr-renewal-reminder' // Page
);

//Disabled in v1.5
/*
     add_settings_section(
         'wsrr_misc_setting_section', // ID
         'Misc Settings', // Title
         array( $this, 'print_section_info' ), // Callback
         'wsrr-renewal-reminder' // Page
     );
*/

    add_settings_field(
          'wsrr_debug', // ID
          'Debug', // Title
          array( $this, 'wsrr_debug_callback' ), // Callback
          'wsrr-renewal-reminder', // Page
          'wsrr_sca_setting_section' // Section
      );
//Disabled in v1.5
/*
      add_settings_field(
            'wsrr_debug_subscription_id', // ID
            'Test Subscription ID ', // Title
            array( $this, 'wsrr_debug_subscription_id_callback' ), // Callback
            'wsrr-renewal-reminder', // Page
            'wsrr_misc_setting_section' // Section
        );

*/

 }

 /**
  * Sanitize each setting field as needed
  *
  * @param array $input Contains all settings fields as array keys
  */
 public function sanitize( $input )
 {

     //$new_input = array();
     if( isset( $input['renewal_days'] ) )
         $new_input['renewal_days'] = absint( $input['renewal_days'] );

     if( isset( $input['renewal_days_offset'] ) )
         $new_input['renewal_days_offset'] = absint( $input['renewal_days_offset'] );

     if( isset( $input['email_bcc'] ) )
         $new_input['email_bcc'] = sanitize_text_field( $input['email_bcc'] );

     if( isset( $input['email_cc'] ) )
         $new_input['email_cc'] = sanitize_text_field( $input['email_cc'] );

    if( isset( $input['email_subject'] ) )
         $new_input['email_subject'] = $input['email_subject'];

    if( isset( $input['email_title'] ) )
         $new_input['email_title'] = $input['email_title'];

    if( isset( $input['email_body'] ) )
         $new_input['email_body'] = $input['email_body'];

    if( isset( $input['email_body_cheque'] ) )
         $new_input['email_body_cheque'] = $input['email_body_cheque'];

    if( isset( $input['wsrr_enable'] ) )
         $new_input['wsrr_enable'] = sanitize_text_field( $input['wsrr_enable'] );

    if( isset( $input['skip_overdue'] ) )
          $new_input['skip_overdue'] = sanitize_text_field( $input['skip_overdue'] );

    if( isset( $input['exclude_year'] ) )
          $new_input['exclude_year'] = sanitize_text_field( $input['exclude_year'] );

    if( isset( $input['exclude_month'] ) )
          $new_input['exclude_month'] = sanitize_text_field( $input['exclude_month'] );

    if( isset( $input['exclude_week'] ) )
          $new_input['exclude_week'] = sanitize_text_field( $input['exclude_week'] );

    if( isset( $input['exclude_day'] ) )
          $new_input['exclude_day'] = sanitize_text_field( $input['exclude_day'] );

    if( isset( $input['wsrr_debug'] ) )
         $new_input['wsrr_debug'] = sanitize_text_field( $input['wsrr_debug'] );
// Disabled v1.5
/*
    if( isset( $input['wsrr_debug_subscription_id'] ) )
         $new_input['wsrr_debug_subscription_id'] = absint(( $input['wsrr_debug_subscription_id'] ));
*/
     return $new_input;
 }



 /**
  * Print the Section text
  */
 public function print_section_info()
 {
echo '
 <style>
     #email_cc, #email_bcc, #email_subject, #email_title  {
        width:100%;
        max-width:500px;
     }
 </style>
     ';
 }
 public function print_section_footer()
 {
    // Added v1.5 
    echo '<p><a href="'.get_site_url().'/wp-admin/tools.php?page=action-scheduler&status=pending&s=wsrr_renewal_reminder_email&action=-1&action2=-1&paged=1">Click here to see all scheduled emails by this plugin</a></p>';
    echo '<p>Below is an example of a Subscription Renewal action, this action sends the email to the customer when triggered.<br>';
    echo 'If this action is "<b>Canceled</b>" it will re-appear the next time the "<b>wsrr_renewal_reminder_scheduler</b>" runs. <br>';
    echo 'To send the renewal email early for a specific subscription, just click run on the "<b>wsrr_renewal_reminder_email</b>" action.</p>';
    echo '<img style ="max-height: 160px;" src="'.wsrr_plugin_dir_url().'images/wsrr-example-order-action.png">';
    echo '<h3>Emails not being sent out?</h3>';
    echo '<p>Please check if the Wordpress CRON is active and runnong on your server. <a href="https://wordpress.org/plugins/wp-cron-status-checker/" target="_blank" >This Plugin can be used to check your CRON</a> </p>';

 }

 /**
  * Get the settings option array and print one of its values
  */
 public function wsrr_enable_callback() {
   $value = !empty($this->options['wsrr_enable'])?$this->options['wsrr_enable']:'';
     print(
         '<input type="checkbox" id="wsrr_enable" name="wsrr_option[wsrr_enable]" value="1" '.checked( 1, $value, false ).'/>'
       );
       print(
          '<label for="checkbox_element"> <br>Please set "Notify Customer" and "Scan orders" before enabling.<br>Once enabled emails will be scheduled according to your settings.</label>'
        );
        print(
            '<br><p><b>IMPORTANT:</b><br> In order for the emails to be sent out, the renewal email template needs to be configured and "Enable this email notification" needs to be checked by clicking the link below.<br> <a href="'.get_site_url().'/wp-admin/admin.php?page=wc-settings&tab=email&section=wc_customer_subscription_renewal_reminder">Click Here To Edit Email Template</a>
            <br>To send a TEST email, just go to the link above and follow the instructions under "Send a Test Renewal Email".</p>'
          );
 }



 /**
  * Get the settings option array and print one of its values
  */
 public function exclude_year_callback() {
   $value = !empty($this->options['exclude_year'])?$this->options['exclude_year']:'';
     print(
         '<input type="checkbox" id="exclude_year" name="wsrr_option[exclude_year]" value="1" '.checked( 1, $value, false ).'/>'
       );
       //print(
        //  '<label for="checkbox_element">Exclude Yearly Renewals</label>'
        //);
 }

  /**
  * Get the settings option array and print one of its values
  */
  public function exclude_day_callback() {
    $value = !empty($this->options['exclude_day'])?$this->options['exclude_day']:'';
      print(
          '<input type="checkbox" id="exclude_day" name="wsrr_option[exclude_day]" value="1" '.checked( 1, $value, false ).'/>'
        );
       // print(
           //'<label for="checkbox_element">Exclude Daily Renewals</label>'
       //  );
  }


   /**
  * Get the settings option array and print one of its values
  */
 public function exclude_month_callback() {
    $value = !empty($this->options['exclude_month'])?$this->options['exclude_month']:'';
      print(
          '<input type="checkbox" id="exclude_month" name="wsrr_option[exclude_month]" value="1" '.checked( 1, $value, false ).'/>'
        );
       // print(
           //'<label for="checkbox_element">Exclude Monthley Renewals</label>'
       //  );
  }

 /**
  * Get the settings option array and print one of its values
  */
 public function exclude_week_callback() {
   $value = !empty($this->options['exclude_week'])?$this->options['exclude_week']:'';
     print(
         '<input type="checkbox" id="exclude_week" name="wsrr_option[exclude_week]" value="1" '.checked( 1, $value, false ).'/>'
       );
      // print(
        //  '<label for="checkbox_element">Exclude Weekley Renewals</label>'
        //);
 }

 /**
  * Get the settings option array and print one of its values
  */
 public function skip_overdue_callback() {
   $value = !empty($this->options['skip_overdue'])?$this->options['skip_overdue']:'';
     print(
         '<input type="checkbox" id="skip_overdue" name="wsrr_option[skip_overdue]" value="1" '.checked( 1, $value, false ).'/>'
       );
       print(
          '<label for="checkbox_element"> Skip reminder emails that are overdue and past the X days reminder.</label>'
        );
 }

 /**
  * Get the settings option array and print one of its values
  */

 public function wsrr_debug_callback() {
    $value = !empty($this->options['wsrr_debug'])?$this->options['wsrr_debug']:'';
     print(
         '<input type="checkbox" id="wsrr_debug" name="wsrr_option[wsrr_debug]" value="1" '.checked( 1, $value, false ).'/>'
       );
     print(
        '<label for="checkbox_element"> Checking this will enable logging in the woocommerce debug log. <p><a href="'.get_site_url().'/wp-admin/admin.php?page=wc-status&tab=logs">Click Here</a> to go to the logs page. Once there select the log starting with "woocommerce-subscriptions-renewal-reminder-" and click the view button.</label>'
      );
 }
 

 /**
  * Get the settings option array and print one of its values
  */
 // Disabled v1.5
/* 
 public function wsrr_debug_subscription_id_callback() {
   // $hook = 'wsrr_renewal_reminder_email';
   // $args = array(
   //   //'renewal_date'    => gmdate("Y-m-d\ H:i:s", $trigger_date)
   // );
   // $group = 'wsrr';
   // $value = ActionScheduler::store()->find_action( $hook, $params,$group );
   // var_dump($value);

     printf(
         '<input type="text" id="wsrr_debug_subscription_id" name="wsrr_option[wsrr_debug_subscription_id]" value="%s" />
          <p class="description">If this is set, emails will ONLY be sent for this specific subscription ID</p><br>
          <p><a href="'.get_site_url().'/wp-admin/tools.php?page=action-scheduler&status=pending&s=wsrr_renewal_reminder_email&action=-1&action2=-1&paged=1">Click here to see all scheduled emails by this plugin</a></p>',
         isset( $this->options['wsrr_debug_subscription_id'] ) ? esc_attr( $this->options['wsrr_debug_subscription_id']) : ''
     );
 }
*/



 /**
  * Get the settings option array and print one of its values
  */
 public function wsrr_renewal_days_callback() {
     printf(
         '<input type="text" id="renewal_days" name="wsrr_option[renewal_days]" value="%s" />
          <p class="description">Notify customer X ( days ) before a subscription is due for renewal</p>
         ',
         isset( $this->options['renewal_days'] ) ? esc_attr( $this->options['renewal_days']) : ''
     );
 }

 /**
  * Get the settings option array and print one of its values
  */
 public function wsrr_renewal_days_offset_callback()
 {
     printf(
         '<input type="text" id="renewal_days_offset" name="wsrr_option[renewal_days_offset]" value="%s" />
          <p class="description">A safe value is 30, if your site has over 1000 subscribers lower this value to 10<br> 
          <b>How This Works</b>:<br> If the renewal reminder email is set to 15 (days) and this option is set to 10 (days), all orders expiring 25 days from today will be scanned and emails scheduled to be sent 15 days before renewal is due.</p>
         ',
         isset( $this->options['renewal_days_offset'])  ? esc_attr( $this->options['renewal_days_offset']) : ''
     );
 }

 /**
  * Get the settings option array and print one of its values
  */
 public function wsrr_cc_callback()
 {
     printf(
         '
         <input type="text" id="email_cc" name="wsrr_option[email_cc]" value="%s" />
         <p class="description">Comma seperated. Ex: email@domain.com, email2@domain.com </p>
         ',
         isset( $this->options['email_cc'] ) ? esc_attr( $this->options['email_cc']) : ''
     );
 }

 /**
  * Get the settings option array and print one of its values
  */
 public function wsrr_bcc_callback()
 {
     printf(
         '
         <input type="text" id="email_bcc" name="wsrr_option[email_bcc]" value="%s" />
         <p class="description">Comma seperated. Ex: email@domain.com, email2@domain.com </p>
         ',
         isset( $this->options['email_bcc'] ) ? esc_attr( $this->options['email_bcc']) : ''
     );
 }

 /**
  * Get the settings option array and print one of its values
  */
 public function email_subject_callback()
 {
   printf(
       '
       <input type="text" id="email_subject" name="wsrr_option[email_subject]" value="%s" />
       <p class="description">Subject of the renewal email sent to subscribers</p>
       ',
       isset( $this->options['email_subject'] ) ? esc_attr( $this->options['email_subject']) : ''
   );
 }

 /**
  * Get the settings option array and print one of its values
  */
 public function email_title_callback()
 {
   printf(
       '
       <input type="text" id="email_title" name="wsrr_option[email_title]" value="%s" />
       <p class="description">Title of the renewal email sent to subscribers</p>
       ',
       isset( $this->options['email_title'] ) ? esc_attr( $this->options['email_title']) : ''
   );
 }

 /**
  * Get the settings option array and print one of its values
  */
 public function email_body_callback()
 {
    $email_body = $this->options['email_body'];
    echo wp_editor( $email_body, 'wsrr_option_email_body', array('textarea_name' => 'wsrr_option[email_body]') );
 }

 /**
  * Get the settings option array and print one of its values
  */
 public function email_body_cheque_callback()
 {
    $email_body = $this->options['email_body_cheque'];
    echo wp_editor( $email_body, 'wsrr_option[email_body_cheque]', array('textarea_name' => 'wsrr_option[email_body_cheque]') );
 }

}

if( is_admin() )
 $wsrr_settings_page = new WSRRSettingsPage();
